#include <stdio.h>
#include <iostream>
#include <unistd.h>
#include "jbcomp.h"

using namespace std;

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+ FUNC : JB_Decomp
//+ 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static bool JB_Decomp(FILE *fp_write, FILE *fp_read)
{
	int i;
	unsigned char Data;
	unsigned char tmpLen;
	size_t Len, writeLen;
	unsigned char DataBuf[DATA_BUF_SIZE];
	
	while (1) {
		if(fread(&Data, 1, 1, fp_read) != 1) {
			if(feof(fp_read) == 0)
				return false;
			break;
		}
		Len = 1;

		if(Data == C_KEY){
			if (fread(&Data, 1, 1, fp_read) != 1) {
				if(feof(fp_read) == 0)
					return false;
				break;
			}

			if (fread(&tmpLen, 1, 1, fp_read) != 1)
				return false;
			Len = tmpLen & DECODE_MASK;

			for(i = 1; ; i++) {
				if (tmpLen & 0x80) {
					if (fread(&tmpLen, 1, 1, fp_read) != 1)
						return false;
					Len |= (tmpLen & DECODE_MASK) << (i * 7);
				} else
					break;
			}
		}

		do {
			if (Len > DATA_BUF_SIZE)
				writeLen = DATA_BUF_SIZE;
			else
				writeLen = Len;

			memset(DataBuf, (int)Data, writeLen);
			if (fwrite(DataBuf, 1, writeLen, fp_write) != writeLen)
				return false;

			Len -= writeLen;
		} while (Len > 0);
	}

	return true;
}

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+ FUNC : JB_Decompession
//+ 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
bool JB_Decompression(const char *writeFileName,
		      const char *readFileName)
{
	FILE *fp_read = stdin;
	FILE *fp_write = stdout;
	bool ret;

	if (writeFileName) {
		fp_write = fopen(writeFileName,"w");
		if (fp_write == NULL) {
			cerr << writeFileName << ": No such file" << endl;
			return false;
		}
	}

	if (readFileName) {
		fp_read = fopen(readFileName,"r");
		if (fp_read == NULL) {
			if (writeFileName)
				fclose(fp_read);
			cerr << readFileName << ": No such file" << endl;
			return false;
		}
	}

	ret = JB_Decomp(fp_write, fp_read);
	if (ret == false) {
		if (ferror(fp_read))
			cerr << readFileName << ": File in wrong format" << endl;
		if (ferror(fp_write))
			cerr << "Failed to compress file, please check permission" << endl;
	}

	if(writeFileName)
		fclose(fp_write);

	if(readFileName)
		fclose(fp_read);

	return ret;
}

void usage()
{
	cout << "Usage: jbcomp [FILE] [OPTION]" << endl;
	cout << "Compress XSVF file sending to JTAG-Blazer" << endl;
	cout << "With no FILE, read standard input" << endl;
	cout << " -o,\toutput file name" << endl;
	cout << " -h,\tshow this message" << endl;
}

int main(int argc, char* argv[])
{
	int opt;
	char* outFileName = NULL;
	char* inFileName = NULL;

	while ((opt = getopt(argc, argv, "ho:")) != -1) {
		switch(opt){
		case 'o':
			outFileName = optarg;
			break;
		case 'h':
			usage();
			return 0;
		default:
			usage();
			return -1;
		}
	}
			
	if (optind < argc) {
		if (optind < (argc - 1)) {
			cerr << argv[0] << ": Invalid command format" << endl;
			usage();
			return -1;
		}
		inFileName = argv[optind];
	}
		
	if (!JB_Decompression(outFileName, inFileName))
		return -1;

	return 0;
}
