#include <iostream>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <math.h>
#include "jblib.h"

using namespace std;

void usage()
{
	cout << "Usage: jbgetspeed [IP ADDRESS] [OPTION]" << endl;
	cout << "Show JTAG line clock speed [MHz] of JTAG-Blazer" << endl;
	cout << " -h,\tshow this message" << endl;
}

int main(int argc, char** argv)
{
	int opt;
	while((opt = getopt(argc, argv, "h")) != -1){
		switch(opt){
		case 'h':
			usage();
			return 0;
		default:
			usage();
			return -1;
		}
	}

	if(optind >= argc){
		cerr << argv[0] << ": IP address is not specified" << endl;
		usage();
		return -1;
	}
	in_addr addr;
	if(!inet_aton(argv[optind], &addr)){
		cerr << argv[0] << ": Invalid IP address" << endl;
		usage();
		return -1;
	}

	JBErrorCode errCode;
	CJBLib lib(addr.s_addr);
	unsigned long speed;
	errCode = lib.TransferRateRequest(&speed);

	if(errCode == JTAG_NO_ERROR){
		double realSpeed = 12.90 / (pow(2.0, (double)speed));
		printf("%5.2f\n", realSpeed);
	}

	return  (int)errCode;
}
