//////////////////////////////////////////////////////////////////////
// file name : JTAGLib.h
// summary   : header file of class for JTAG BLAZER
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CJBLIB_H__E03793E7_8DD5_45CE_A48D_84ECB7F0143D__INCLUDED_)
#define AFX_CJBLIB_H__E03793E7_8DD5_45CE_A48D_84ECB7F0143D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>
#include <string>

#define MAX_FILE_NAME_LEN		256
#define MAX_VERSION_NAME_LEN	1024

#define RCV_BUF_SIZE 1024

using namespace std;

struct FlashFileData
{
	bool littleBear;
	bool compressed;
	time_t recordTime;
	unsigned long fileSize;
	char fileName[MAX_FILE_NAME_LEN + 1];
	unsigned long dataLen;
};

/* will be removed -- yashi */
#include "jberror.h"
#include "jbdata.h"

class CJBLib
{
public:
	enum MaxSizeType
	{
		CONFIG_DATA_SIZE	= 0,
		UPDATE_PROGRAM_SIZE	= 1,
	};

	CJBLib(unsigned long connectIP = 0);
	virtual ~CJBLib();

	JBErrorCode TargetWriteRequest(
				const char* fileName,
				bool littleBear,
				void (*ProgressNotify)(int status, int param)
				);
	JBErrorCode FlashWriteRequest(
				const char* fileName,
				const FlashFileData* fileData,
				void (*ProgressNotify)(int status, int param)
				);
	JBErrorCode UpdateProgramRequest(
				const char* fileName,
				void (*ProgressNotify)(int status, int param)
				);
	JBErrorCode DeviceListRequest(
				vector<unsigned int>* deviceList
				);
	JBErrorCode MaxUpdateProgramSizeRequest(
				unsigned long* maxSize
				);
	JBErrorCode MaxConfigDataSizeRequest(
				unsigned long* maxSize
				);
	JBErrorCode TransferRateRequest(
				unsigned long *transferRate
				);
	JBErrorCode ChangeTransferRateRequest(
				unsigned long transferRate
				);
	JBErrorCode FlashFileDataRequest(
				FlashFileData* fileData
				);
	JBErrorCode FirmwareVersionRequest(
				char* versionName
				);
	JBErrorCode FlashReadRequest(
				const char* fileName
				);
	JBErrorCode RecvFileData(
				const char* fileName
				);
	bool PrepareSocket(unsigned long targetIP = 0, unsigned short portNo = 0);
	bool CancelRequest();
	
	bool IsConnected()
		{ return (m_socket < 0)? false : true; };
	unsigned long GetConnectedIP() const
		{ return m_targetIP; };
	void CloseSocket();

private:
	int m_socket;
	unsigned int	m_sequence;
	unsigned short	m_portNo;
	unsigned long	m_targetIP;

protected:
	JBErrorCode GetMaxDataSize(
				MaxSizeType type,
				unsigned long* maxSize
				);
	JBErrorCode SendFlashFileData(
				const FlashFileData* fileData
				);
	JBErrorCode RecvFlashFileData(
				FlashFileData* fileData
				);
	JBErrorCode SendFileData(
				const char* fileName,
				JBDataID waitID,
				JBDataID notifyID,
				void (*NotifyFunc)(int status, int param)
	);
	unsigned int CalculateCheckSum(
				unsigned char* data,
				size_t size
				) const;
	unsigned long GetProtcolVersion(
				JBDataID id
				);
	bool SendHeader(
				JBDataID id
				);
	bool ReceiveHeader(
				JBDataHeader* header
				);
	JBErrorCode ReceiveResponse(
				JBDataID id
				);
	bool SendLongData(
				unsigned long sendData
				);
	bool ReceiveLongData(
				unsigned long* recvData
				);
	unsigned long GetNewSequenceNo();
};

#endif // !defined(AFX_CJBLIB_H__E03793E7_8DD5_45CE_A48D_84ECB7F0143D__INCLUDED_)
